﻿using EnvDTE80;
using Ora.API;

namespace GroupingSample
{
    /// <summary>
    /// An example grouping for people who write classes with a lot of members!
    /// 
    /// Demonstrates how to derive from MapGrouping: simply specify the grouping
    /// name when calling the base constructor, and override Map.
    /// </summary>
    public class Alphabetical : MapGrouping
    {
        public Alphabetical()
            : base("Alphabetical") { }
        
        /// <summary>
        /// This function examines a single member and returns the name of the
        /// group it should be placed in. In this example we take the first
        /// letter character of the member's name and return it as a string.
        /// </summary>
        /// <param name="member">A members to be sorted into a group</param>
        /// <param name="parent">The type this member belongs to (not used here)</param>
        /// <returns>A string giving the name to group this member under</returns>
        protected override string Map(CodeElement2 member, CodeElement2 parent)
        {
            string name = member.Name;

            for (int n = 0; n < name.Length; n++)
            {
                if (char.IsLetter(name[n]))
                    return new string(name[n], 1).ToUpper();
            }

            if (string.IsNullOrEmpty(name))
                return "(No Name)";

            return name.Substring(0, 1); // non-alphabetic name!
        }
    }
}
