using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using EnvDTE;
using EnvDTE80;
using Ora.API;
using Reflect;

namespace Ora
{
    public static class CodeElementExtensions
    {
        private static readonly Dictionary<vsCMElement, IMemberKind> _kinds =
            Components.Construct<IMemberKind>().ToDictionary(m => m.Kind);

        public static IEnumerable<IMemberKind> MemberKinds
        {
            get { return _kinds.Values; }
        }

        private class NullKind : IMemberKind
        {
            public vsCMElement Kind { get; set; }

            public string Name
            {
                get { return Kind.ToString(); }
            }

            public Bitmap Image
            {
                get { return null; }
            }

            public string GetCaption(CodeElement2 element)
            {
                return element.Name;
            }

            public vsCMAccess GetAccess(CodeElement2 element)
            {
                return vsCMAccess.vsCMAccessDefault;
            }

            public CodeInterface2 GetInterface(CodeElement2 element, CodeElement2 parent)
            {
                return null;
            }
        }

        public static IMemberKind GetMemberKind(this CodeElement2 elem)
        {
            IMemberKind kind;
            if (_kinds.TryGetValue(elem.Kind, out kind))
                return kind;

            return new NullKind { Kind = elem.Kind };
        }
    }
}
