using System.Collections.Generic;
using EnvDTE80;

namespace Ora.API
{
    /// <summary>
    /// So-called because it has no name (see IGrouping which adds naming), but apart from
    /// that it defines all the capabilities of a grouping. Specifically, groupings can be
    /// chained together by assigning to their NextInChain properties, and they can generate a 
    /// tree of nodes from information about a class, struct or interface.
    /// </summary>
    public interface IAnonymousGrouping
    {
        /// <summary>
        /// The next grouping in the chain. For any grouping selected in the UI of Ora,
        /// this is guaranteed to be non-null, because Ora provides a terminating grouping
        /// that just dumps out the remaining members in alphabetical order.
        /// </summary>
        IAnonymousGrouping NextInChain { get; set; }

        /// <summary>
        /// Generates nodes from the supplied information, which consists of a set of
        /// members (that is: functions, properties, fields, etc.) and also the type to
        /// which the members all belong (either a class, struct or interface). 
        /// 
        /// It should of course be possible to obtain the parent type from any of the members, 
        /// but there seem to be bugs in Visual Studio in that area. The resulting nodes
        /// are created by calling methods on IResultsTreeNode. When a group of members
        /// has been identified, they should be passed onto the next grouping by calling
        /// NextInChain.Generate().
        /// 
        /// It is best to avoid walking the grouping chain within this function. Just call 
        /// NextInChain.Generate() and don't try sniffing around in NextInChain.NextInChain.NextInChain, etc.
        /// </summary>
        /// <param name="node">The tree node to fill with results</param>
        /// <param name="members">The members of the type being viewed</param>
        /// <param name="parent">The type being viewed</param>
        void Generate(IResultsTreeNode node, IEnumerable<CodeElement2> members, CodeElement2 parent);
    }
}