using System.Drawing;
using EnvDTE;
using EnvDTE80;

namespace Ora.API
{
    /// <summary>
    /// A component that encapsulates a kind of member. Examples are methods,
    /// properties, fields, etc. Given that this set rarely expands, this is
    /// probably not an interesting area for new component development in Ora.
    /// </summary>
    public interface IMemberKind
    {
        /// <summary>
        /// The kind of member this component applies to.
        /// </summary>
        vsCMElement Kind { get; }

        /// <summary>
        /// The name to use for this kind of member in the GUI.
        /// </summary>
        string Name { get; }

        /// <summary>
        /// The icon for this type of member.
        /// </summary>
        Bitmap Image { get; }

        /// <summary>
        /// Generates a suitable treenode label for an instance of this kind.
        /// So for methods it might include parameters, otherwise it's just
        /// going to be the name.
        /// </summary>
        /// <param name="element">The member instance</param>
        /// <returns>The caption</returns>
        string GetCaption(CodeElement2 element);

        /// <summary>
        /// Returns the access level (public, private, etc.) for an instance
        /// of this kind.
        /// </summary>
        /// <param name="element">The member instance</param>
        /// <returns>The access level</returns>
        vsCMAccess GetAccess(CodeElement2 element);

        /// <summary>
        /// Returns the interface that this member implements a member of.
        /// Both classes and structs support this. Everything else always
        /// returns null.
        /// </summary>
        /// <param name="element">The member instance</param>
        /// <param name="parent">The type the member belongs to</param>
        /// <returns>The interface, or null</returns>
        CodeInterface2 GetInterface(CodeElement2 element, CodeElement2 parent);
    }
}