using Microsoft.Win32;

namespace Ora
{
    public static class Configuration
    {
        private static RegistryKey Key
        {
            get { return Registry.CurrentUser.CreateSubKey("Software\\DanielEarwicker\\Ora"); }
        }

        private static bool _cacheReady;

        private static string _groupings;
        private static bool _hideMethodUnderscores;
        private static int _refreshTimeout;

        private static void ReadCache()
        {
            if (_cacheReady)
                return;

            _groupings = (string) Key.GetValue("Grouping", "Interface, Polymorphism, Access");
            _hideMethodUnderscores = ((string) (Key.GetValue("HideMethodUnderscores", "0"))) == "1";

            if (!int.TryParse((string)Key.GetValue("RefreshTimeout"), out _refreshTimeout))
                _refreshTimeout = 500;

            _cacheReady = true;
        }

        public static string Groupings
        {
            get { ReadCache(); return _groupings; }
            set { _groupings = value; Key.SetValue("Grouping", value); }
        }

        public static bool HideMethodUnderscores
        {
            get { ReadCache(); return _hideMethodUnderscores; }
            set { _hideMethodUnderscores = value; Key.SetValue("HideMethodUnderscores", value ? "1" : "0"); }
        }

        public static int RefreshTimeout
        {
            get { ReadCache(); return _refreshTimeout; }
            set { _refreshTimeout = value; Key.SetValue("RefreshTimeout", value.ToString()); }
        }
    }
}