﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using Ora.API;

namespace Ora
{
    public partial class ConfigurationDialog : Form
    {
        public ConfigurationDialog()
        {
            InitializeComponent();

            buttonDown.Enabled = false;
            buttonUp.Enabled = false;

            checkedListBoxGroupings.Focus();
        }

        private List<IGrouping> _groupings, _selected;

        public List<IGrouping> Groupings
        {
            get { return _groupings; }
            set
            {
                _groupings = value;
                Init();
            }
        }

        public List<IGrouping> Selected
        {
            get { return _selected; }
            set
            {
                _selected = value;
                Init();
            }
        }

        public bool HideMethodUnderscores
        {
            get { return checkBoxMethodUnderscores.Checked; }
            set { checkBoxMethodUnderscores.Checked = value; }
        }

        public int RefreshTimeout
        {
            get { return int.Parse(textBoxTimeout.Text); }
            set { textBoxTimeout.Text = value.ToString(); }
        }

        private void Init()
        {
            if (_groupings != null && _selected != null)
            {
                foreach (IGrouping grouping in _selected)
                {
                    int index = checkedListBoxGroupings.Items.Add(grouping.Name);
                    checkedListBoxGroupings.SetItemChecked(index, true);
                }

                foreach (IGrouping grouping in _groupings)
                {
                    if (!_selected.Contains(grouping))
                        checkedListBoxGroupings.Items.Add(grouping.Name);
                }
            }
        }

        public void SwapItems(int first)
        {
            object item = checkedListBoxGroupings.Items[first];
            bool check = checkedListBoxGroupings.CheckedItems.Contains(item);
            checkedListBoxGroupings.Items.RemoveAt(first);
            checkedListBoxGroupings.Items.Insert(first + 1, item);
            checkedListBoxGroupings.SetItemChecked(first + 1, check);
        }

        private void buttonUp_Click(object sender, EventArgs e)
        {
            int sel = checkedListBoxGroupings.SelectedIndex;
            if ((sel > 0) && (checkedListBoxGroupings.Items.Count > 1))
            {
                SwapItems(sel - 1);
                checkedListBoxGroupings.SelectedIndex = sel - 1;
                EnableButtons();
            }
        }

        private void buttonDown_Click(object sender, EventArgs e)
        {
            int sel = checkedListBoxGroupings.SelectedIndex;
            if ((sel < checkedListBoxGroupings.Items.Count - 1) &&
                (checkedListBoxGroupings.Items.Count > 1))
            {
                SwapItems(sel);
                checkedListBoxGroupings.SelectedIndex = sel + 1;
                EnableButtons();
            }
        }

        private void checkedListBoxGroupings_SelectedIndexChanged(object sender, EventArgs e)
        {
            EnableButtons();
        }

        private void buttonOK_Click(object sender, EventArgs e)
        {
            int timeoutValidate;
            if (!int.TryParse(textBoxTimeout.Text, out timeoutValidate) || (timeoutValidate < 100))
            {
                MessageBox.Show(this, "Timeout must be an integer, and at least 100");
                return;
            }

            List<IGrouping> newGroups = new List<IGrouping>();

            foreach (string item in checkedListBoxGroupings.Items)
            {
                if (checkedListBoxGroupings.CheckedItems.Contains(item))
                    newGroups.Add(_groupings.Where(grouping => grouping.Name == item).Single());
            }

            _selected = newGroups;

            DialogResult = DialogResult.OK;
        }

        private void EnableButtons()
        {
            int sel = checkedListBoxGroupings.SelectedIndex;

            buttonUp.Enabled = ((sel > 0) && (checkedListBoxGroupings.Items.Count > 1));
            buttonDown.Enabled = ((sel < checkedListBoxGroupings.Items.Count - 1) &&
                                  (checkedListBoxGroupings.Items.Count > 1));   
        }
    }
}
