using System.Drawing;
using System.Text;
using EnvDTE;
using EnvDTE80;

namespace Ora.MemberKinds
{
    public class MemberFunction : MemberKindBase<CodeFunction2>
    {
        public MemberFunction()
            : base(vsCMElement.vsCMElementFunction, "function") { }

        protected override string GetCaption(CodeFunction2 element)
        {
            string funcSig = GetFunctionSignature(element, false);
            if (Configuration.HideMethodUnderscores)
                funcSig = funcSig.Replace('_', ' ');
            return funcSig;
        }

        protected override vsCMAccess GetAccess(CodeFunction2 element)
        {
            return element.Access;
        }

        public override Bitmap Image
        {
            get { return _bitmap; }
        }

        private static readonly Bitmap _bitmap = Icons.Properties.Resources.Method;

        protected override CodeInterface2 GetInterface(CodeFunction2 func, CodeElement2 parent)
        {
            return ((CodeElement2) func).GetAssociatedInterface(parent, mem =>
                 {
                     CodeFunction2 funcMember = mem as CodeFunction2;
                     if (funcMember != null)
                         return GetFunctionSignature(funcMember, true);
                     return null;
                 });
        }

        public static string GetFunctionSignature(CodeFunction2 func, bool fullTypes)
        {
            StringBuilder label = new StringBuilder();

            if (fullTypes)
            {
                label.Append(func.Type.AsFullName);
                label.Append(' ');
            }

            label.Append(EnvDTEExtensions.TrimType(func.Name));
            
            label.Append('(');

            bool first = true;
            foreach (CodeParameter2 param in func.Parameters)
            {
                if (first)
                    first = false;
                else
                    label.Append(", ");

                if (fullTypes)
                    label.Append(param.Type.AsFullName);
                else
                    label.Append(EnvDTEExtensions.TrimType(param.Type.AsString));
            }

            label.Append(')');

            if (!fullTypes)
            {
                string returnType = EnvDTEExtensions.TrimType(func.Type.AsFullName);
                if (returnType.Length != 0)
                {
                    label.Append(" : ");
                    label.Append(returnType);
                }
            }

            return label.ToString();
        }
    }
}