using System;
using System.Diagnostics;
using System.Drawing;
using System.Windows.Forms;
using EnvDTE80;
using Ora.API;

namespace Ora
{
    internal class ResultsTreeNode : IResultsTreeNode
    {
        private readonly TreeNodeCollection _collection;
        private readonly Images _images;
        private readonly Func<bool> _shouldAbort;

        public ResultsTreeNode(TreeNodeCollection collection, Images images, Func<bool> shouldAbort)
        {
            _collection = collection;
            _shouldAbort = shouldAbort;
            _images = images;
        }

        public void Add(CodeElement2 member)
        {
            IMemberKind kind = member.GetMemberKind();

            int image = _images[kind.Image];
            string caption = kind.GetCaption(member);

            _collection.Add(caption, caption, image, image).Tag = member;
        }

        public IResultsTreeNode Add(string label, Bitmap image)
        {
            if (image == null)
                image = Images.DefaultImage;

            int index = _images[image];
            return new ResultsTreeNode(_collection.Add(label, label, index, index).Nodes, _images, _shouldAbort);
        }

        private static bool Empty(TreeNodeCollection nodes)
        {
            if (nodes.Count == 0)
                return true;

            TreeNode child = nodes[0];

            do
            {
                bool empty = Empty(child.Nodes);
                TreeNode remove = (child.Tag == null && empty) ? child : null;

                child = child.NextNode;

                if (remove != null)
                    nodes.Remove(remove);
            } 
            while (child != null);

            return nodes.Count == 0;
        }

        public bool Empty()
        {
            return Empty(_collection);
        }

        public bool ShouldAbort
        {
            get { return _shouldAbort(); }
        }
    }
}