﻿using System;
using System.IO;
using System.Windows.Forms;
using Microsoft.Win32;

namespace OraSetup
{
    public partial class FormInstall : Form
    {
        public FormInstall()
        {
            InitializeComponent();

            // Be uninstall or setup, depending on .exe filename
            if (Path.GetFileName(GetType().Assembly.Location).ToLower() == Strings.Uninstall.ToLower())
            {
                labelPrompt.Text = Strings.RemovePrompt;
                buttonGo.Text = Strings.RemoveButton;
                _uninstall = true;
            }
            else
            {
                labelPrompt.Text = Strings.InstallPrompt;
                buttonGo.Text = Strings.InstallButton;
            }
        }

        private string _destinationDir;
        private string _sourceDir;
        private readonly bool _uninstall;

        private bool Copy(string sourceFile, string destinationFile)
        {
            for (;;)
            {
                string source = Path.Combine(_sourceDir, sourceFile),
                       destination = Path.Combine(_destinationDir, destinationFile);

                try
                {
                    string destDir = Path.GetDirectoryName(destination);
                    if (!Directory.Exists(destDir))
                        Directory.CreateDirectory(destDir);

                    File.Copy(source, destination, true);
                    return true;
                }
                catch (Exception x)
                {
                    if (MessageBox.Show(this, string.Format(Strings.CopyError, x.Message, source, destination),
                        Strings.Ora, MessageBoxButtons.OKCancel) == DialogResult.Cancel)
                        break;
                }
            }

            return false;
        }

        private static void AttemptDeleteSilent(string path)
        {
            try
            {    
                if (Directory.Exists(path))
                    Directory.Delete(path, true);
                else if (File.Exists(path))
                    File.Delete(path);
            }
// ReSharper disable EmptyGeneralCatchClause
            catch
// ReSharper restore EmptyGeneralCatchClause
            { }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            string oldVersionDir = Path.Combine(
                Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments),
                    "Visual Studio 2008\\Addins");

            AttemptDeleteSilent(Path.Combine(oldVersionDir, Strings.Ora));
            AttemptDeleteSilent(Path.Combine(oldVersionDir, Strings.OraAddin));
    
            _destinationDir = Path.Combine(
                Environment.GetFolderPath(Environment.SpecialFolder.CommonApplicationData),
                    Strings.AddinsSubDir);

            if (_uninstall)
            {
                for (;;)
                {
                    AttemptDeleteSilent(Path.Combine(_destinationDir, Strings.Ora));

                    try
                    {
                        string addinFile = Path.Combine(_destinationDir, Strings.OraAddin);
                        if (File.Exists(addinFile))
                            File.Delete(addinFile);

                        Registry.LocalMachine.DeleteSubKey(Strings.UninstallKey, false);

                        MessageBox.Show(Strings.Success);
                        break;
                    }
                    catch (Exception x)
                    {
                        if (MessageBox.Show(this, string.Format(Strings.DeleteError, x.Message),
                            Strings.Ora, MessageBoxButtons.OKCancel) == DialogResult.Cancel)
                            break;
                    }
                }

                DialogResult = DialogResult.Cancel;
            }
            else
            {
                if (Directory.Exists(_destinationDir))
                    Directory.CreateDirectory(_destinationDir);

                string setupDir = Path.GetDirectoryName(typeof (FormInstall).Assembly.Location);
                _sourceDir = Path.Combine(setupDir, Strings.SetupFiles);
                if (!Directory.Exists(_sourceDir))
                {
                    MessageBox.Show(this, Strings.SourceError);
                    DialogResult = DialogResult.Cancel;
                    return;
                }

                foreach (string fileName in Directory.GetFiles(_sourceDir))
                {
                    string name = Path.GetFileName(fileName);
                    if (!Copy(name, Strings.Ora + "\\" + name))
                    {
                        DialogResult = DialogResult.Cancel;
                        return;
                    }
                }

                if (!Copy(Strings.OraAddin, Strings.OraAddin))
                {
                    DialogResult = DialogResult.Cancel;
                    return;
                }

                RegistryKey key = Registry.LocalMachine.CreateSubKey(Strings.UninstallKey);
                key.SetValue("DisplayName", "Ora - AddIn for Visual Studio 2008");
                key.SetValue("UninstallString",
                             "\"" + Path.Combine(_destinationDir, Path.Combine(Strings.Ora, Strings.Uninstall)) + "\"");

                MessageBox.Show(Strings.Success);
                DialogResult = DialogResult.OK;
            }
        }

        private void button2_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
        }
    }
}
