using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;

namespace Reflect
{
    public static class Extensions
    {
        public static object Construct(this Type type)
        {
            return type.GetConstructor(Type.EmptyTypes).Invoke(null);
        }

        public static bool IsConstructable(this Type type)
        {
            return type.GetConstructor(Type.EmptyTypes) != null;
        }

        public static IEnumerable<Type> GetTypes(this IEnumerable<Assembly> assemblies)
        {
            foreach (Assembly assembly in assemblies)
            {
                foreach (Type type in assembly.GetTypes())
                    yield return type;
            }
        }

        private static bool IsAssemblyFile(string name)
        {
            string ext = Path.GetExtension(name).ToLower().Trim('.');
            return (ext == "exe" || ext == "dll");
        }

        public static IEnumerable<Assembly> GetRelatedAssemblies(this Type type)
        {
            string assemblyPath = Path.GetDirectoryName(type.Assembly.Location);

            foreach (string assemblyFile in Directory.GetFiles(assemblyPath).Where(IsAssemblyFile))
                yield return Assembly.LoadFrom(Path.Combine(assemblyPath, assemblyFile));
        }

    }
}